==> webkit2gtk_2.10.9.orig.tar.xz: webkitgtk-2.10.9/Source/WebCore/workers/WorkerEventQueue.cpp <==
/*
 * Copyright (C) 2010 Google Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 *
 */

#include "config.h"
#include "WorkerEventQueue.h"

#include "DOMWindow.h"
#include "Document.h"
#include "Event.h"
#include "EventNames.h"
#include "ScriptExecutionContext.h"

namespace WebCore {

WorkerEventQueue::WorkerEventQueue(ScriptExecutionContext& context)
    : m_scriptExecutionContext(context)
    , m_isClosed(false)
{
}

WorkerEventQueue::~WorkerEventQueue()
{
    close();
}

class WorkerEventQueue::EventDispatcher
{
public:
    EventDispatcher(PassRefPtr<Event> event, WorkerEventQueue& eventQueue)
        : m_event(event)
        , m_eventQueue(eventQueue)
        , m_isCancelled(false)
    {
    }

    ~EventDispatcher()
    {
        if (m_event)
            m_eventQueue.m_eventDispatcherMap.remove(m_event.get());
    }

    void dispatch()
    {
        if (m_isCancelled)
            return;
        m_eventQueue.m_eventDispatcherMap.remove(m_event.get());
        m_event->target()->dispatchEvent(m_event);
        m_event = nullptr;
    }

    void cancel()
    {
        m_isCancelled = true;
        m_event = nullptr;
    }

private:
    RefPtr<Event> m_event;
    WorkerEventQueue& m_eventQueue;
    bool m_isCancelled;
};

bool WorkerEventQueue::enqueueEvent(PassRefPtr<Event> event)
{
    if (m_isClosed)
        return false;

    EventDispatcher* eventDispatcherPtr = new EventDispatcher(event.get(), *this);
    m_eventDispatcherMap.add(event, eventDispatcherPtr);
    m_scriptExecutionContext.postTask([eventDispatcherPtr] (ScriptExecutionContext&) {
        std::unique_ptr<EventDispatcher> eventDispatcher(eventDispatcherPtr);
        eventDispatcher->dispatch();
    });

    return true;
}

bool WorkerEventQueue::cancelEvent(Event& event)
{
    EventDispatcher* task = m_eventDispatcherMap.take(&event);
    if (!task)
        return false;
    task->cancel();
    return true;
}

void WorkerEventQueue::close()
{
    m_isClosed = true;
    for (auto& dispatcher : m_eventDispatcherMap.values())
        dispatcher->cancel();
    m_eventDispatcherMap.clear();
}

}

==> webkit2gtk_2.20.1.orig.tar.xz: webkitgtk-2.20.1/Source/WebCore/workers/WorkerEventQueue.cpp <==
/*
 * Copyright (C) 2010 Google Inc. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#include "WorkerEventQueue.h"

#include "Event.h"
#include "EventTarget.h"
#include "ScriptExecutionContext.h"

namespace WebCore {

WorkerEventQueue::WorkerEventQueue(ScriptExecutionContext& context)
    : m_scriptExecutionContext(context)
{
}

WorkerEventQueue::~WorkerEventQueue()
{
    close();
}

class WorkerEventQueue::EventDispatcher
{
public:
    EventDispatcher(RefPtr<Event>&& event, WorkerEventQueue& eventQueue)
        : m_event(WTFMove(event))
        , m_eventQueue(eventQueue)
    {
    }

    ~EventDispatcher()
    {
        if (m_event)
            m_eventQueue.m_eventDispatcherMap.remove(m_event.get());
    }

    void dispatch()
    {
        if (m_isCancelled)
            return;
        m_eventQueue.m_eventDispatcherMap.remove(m_event.get());
        m_event->target()->dispatchEvent(*m_event);
        m_event = nullptr;
    }

    void cancel()
    {
        m_isCancelled = true;
        m_event = nullptr;
    }

private:
    RefPtr<Event> m_event;
    WorkerEventQueue& m_eventQueue;
    bool m_isCancelled { false };
};

bool WorkerEventQueue::enqueueEvent(Ref<Event>&& event)
{
    if (m_isClosed)
        return false;

    auto* eventPtr = event.ptr();
    auto eventDispatcher = std::make_unique<EventDispatcher>(WTFMove(event), *this);
    m_eventDispatcherMap.add(eventPtr, eventDispatcher.get());
    m_scriptExecutionContext.postTask([eventDispatcher = WTFMove(eventDispatcher)] (ScriptExecutionContext&) {
        eventDispatcher->dispatch();
    });

    return true;
}

bool WorkerEventQueue::cancelEvent(Event& event)
{
    EventDispatcher* task = m_eventDispatcherMap.take(&event);
    if (!task)
        return false;
    task->cancel();
    return true;
}

void WorkerEventQueue::close()
{
    m_isClosed = true;
    for (auto& dispatcher : m_eventDispatcherMap.values())
        dispatcher->cancel();
    m_eventDispatcherMap.clear();
}

} // namespace WebCore
